==> distribution-6.17.7.zip: drr/regulation/cftc/rewrite/functions/ChangeInNotionalAmountLeg1.java <==
package drr.regulation.cftc.rewrite.functions;

import cdm.base.math.functions.Abs;
import cdm.product.template.Product;
import cdm.product.template.TradableProduct;
import cdm.product.template.TradeLot;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.MapperMaths;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.common.PayoutLegWithAuxiliary;
import drr.regulation.common.ReportableInformation;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.BeforeTradeForEvent;
import drr.regulation.common.functions.IsActionTypeTERM;
import drr.regulation.common.functions.PayoutLeg1;
import drr.standards.iosco.cde.version2.quantity.functions.NotionalAmountFormat;
import drr.standards.iosco.cde.version2.quantity.functions.NotionalAmountLeg1;
import java.math.BigDecimal;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(ChangeInNotionalAmountLeg1.ChangeInNotionalAmountLeg1Default.class)
public abstract class ChangeInNotionalAmountLeg1 implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected Abs abs;
	@Inject protected BeforeTradeForEvent beforeTradeForEvent;
	@Inject protected IsActionTypeTERM isActionTypeTERM;
	@Inject protected NotionalAmountFormat notionalAmountFormat;
	@Inject protected NotionalAmountLeg1 notionalAmountLeg1;
	@Inject protected PayoutLeg1 payoutLeg1;

	/**
	* @param transactionReportInstruction 
	* @param defaultValue 
	* @return change 
	*/
	public BigDecimal evaluate(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue) {
		BigDecimal change = doEvaluate(transactionReportInstruction, defaultValue);
		
		return change;
	}

	protected abstract BigDecimal doEvaluate(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue);

	protected abstract MapperS<? extends PayoutLegWithAuxiliary> payoutBefore(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue);

	protected abstract MapperS<BigDecimal> NotionalAmountBefore(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue);

	protected abstract MapperS<BigDecimal> NotionalAmountAfter(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue);

	public static class ChangeInNotionalAmountLeg1Default extends ChangeInNotionalAmountLeg1 {
		@Override
		protected BigDecimal doEvaluate(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue) {
			BigDecimal change = null;
			return assignOutput(change, transactionReportInstruction, defaultValue);
		}
		
		protected BigDecimal assignOutput(BigDecimal change, TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue) {
			if (areEqual(NotionalAmountBefore(transactionReportInstruction, defaultValue), MapperS.of(defaultValue), CardinalityOperator.All).or(areEqual(NotionalAmountAfter(transactionReportInstruction, defaultValue), MapperS.of(defaultValue), CardinalityOperator.All)).getOrDefault(false)) {
				change = defaultValue;
			} else {
				final Boolean _boolean = isActionTypeTERM.evaluate(transactionReportInstruction);
				if ((_boolean == null ? false : _boolean)) {
					change = NotionalAmountAfter(transactionReportInstruction, defaultValue).get();
				} else {
					change = abs.evaluate(MapperMaths.<BigDecimal, BigDecimal, BigDecimal>subtract(NotionalAmountAfter(transactionReportInstruction, defaultValue), NotionalAmountBefore(transactionReportInstruction, defaultValue)).get());
				}
			}
			
			return change;
		}
		
		@Override
		protected MapperS<? extends PayoutLegWithAuxiliary> payoutBefore(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue) {
			final MapperS<TradableProduct> thenArg = MapperS.of(beforeTradeForEvent.evaluate(transactionReportInstruction)).<TradableProduct>map("getTradableProduct", trade -> trade.getTradableProduct());
			return thenArg
				.mapSingleToItem(item -> MapperS.of(payoutLeg1.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get(), item.<TradeLot>mapC("getTradeLot", tradableProduct -> tradableProduct.getTradeLot()).get(), MapperS.of(transactionReportInstruction).<ReportableInformation>map("getReportableInformation", _transactionReportInstruction -> _transactionReportInstruction.getReportableInformation()).get())));
		}
		
		@Override
		protected MapperS<BigDecimal> NotionalAmountBefore(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue) {
			return MapperS.of(notionalAmountFormat.evaluate(payoutBefore(transactionReportInstruction, defaultValue).get(), defaultValue));
		}
		
		@Override
		protected MapperS<BigDecimal> NotionalAmountAfter(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue) {
			return MapperS.of(notionalAmountLeg1.evaluate(transactionReportInstruction, defaultValue));
		}
	}
}

==> distribution-6.19.3.zip: drr/regulation/cftc/rewrite/functions/ChangeInNotionalAmountLeg1.java <==
package drr.regulation.cftc.rewrite.functions;

import cdm.base.math.functions.Abs;
import cdm.event.common.BusinessEvent;
import cdm.event.qualification.functions.Qualify_ContractFormation;
import cdm.event.qualification.functions.Qualify_Execution;
import cdm.event.workflow.WorkflowStep;
import cdm.product.template.Product;
import cdm.product.template.TradableProduct;
import cdm.product.template.TradeLot;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.expression.MapperMaths;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.cftc.rewrite.trade.reports.EventTypeRule;
import drr.regulation.common.PayoutLegWithAuxiliary;
import drr.regulation.common.ReportableInformation;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.BeforeTradeForEvent;
import drr.regulation.common.functions.IsActionTypeNEWT;
import drr.regulation.common.functions.IsActionTypeTERM;
import drr.regulation.common.functions.PayoutLeg1;
import drr.standards.iosco.cde.version2.quantity.functions.NotionalAmountFormat;
import drr.standards.iosco.cde.version2.quantity.functions.NotionalAmountLeg1;
import drr.standards.iso.EventTypeEnum;
import java.math.BigDecimal;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(ChangeInNotionalAmountLeg1.ChangeInNotionalAmountLeg1Default.class)
public abstract class ChangeInNotionalAmountLeg1 implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected Abs abs;
	@Inject protected BeforeTradeForEvent beforeTradeForEvent;
	@Inject protected EventTypeRule eventTypeRule;
	@Inject protected IsActionTypeNEWT isActionTypeNEWT;
	@Inject protected IsActionTypeTERM isActionTypeTERM;
	@Inject protected NotionalAmountFormat notionalAmountFormat;
	@Inject protected NotionalAmountLeg1 notionalAmountLeg1;
	@Inject protected PayoutLeg1 payoutLeg1;
	@Inject protected Qualify_ContractFormation qualify_ContractFormation;
	@Inject protected Qualify_Execution qualify_Execution;

	/**
	* @param transactionReportInstruction 
	* @param defaultValue 
	* @return change 
	*/
	public BigDecimal evaluate(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue) {
		BigDecimal change = doEvaluate(transactionReportInstruction, defaultValue);
		
		return change;
	}

	protected abstract BigDecimal doEvaluate(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue);

	protected abstract MapperS<? extends BusinessEvent> businessEvent(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue);

	protected abstract MapperS<? extends PayoutLegWithAuxiliary> payoutBefore(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue);

	protected abstract MapperS<BigDecimal> NotionalAmountBefore(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue);

	protected abstract MapperS<BigDecimal> NotionalAmountAfter(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue);

	public static class ChangeInNotionalAmountLeg1Default extends ChangeInNotionalAmountLeg1 {
		@Override
		protected BigDecimal doEvaluate(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue) {
			BigDecimal change = null;
			return assignOutput(change, transactionReportInstruction, defaultValue);
		}
		
		protected BigDecimal assignOutput(BigDecimal change, TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue) {
			if (areEqual(NotionalAmountBefore(transactionReportInstruction, defaultValue), MapperS.of(defaultValue), CardinalityOperator.All).or(areEqual(NotionalAmountAfter(transactionReportInstruction, defaultValue), MapperS.of(defaultValue), CardinalityOperator.All)).getOrDefault(false)) {
				change = defaultValue;
			} else if (ComparisonResult.of(MapperS.of(isActionTypeTERM.evaluate(transactionReportInstruction))).or(ComparisonResult.of(MapperS.of(isActionTypeNEWT.evaluate(transactionReportInstruction))).and(areEqual(MapperS.of(eventTypeRule.evaluate(transactionReportInstruction)), MapperS.of(EventTypeEnum.TRAD), CardinalityOperator.All)).and(ComparisonResult.of(MapperS.of(qualify_ContractFormation.evaluate(businessEvent(transactionReportInstruction, defaultValue).get()))).or(ComparisonResult.of(MapperS.of(qualify_Execution.evaluate(businessEvent(transactionReportInstruction, defaultValue).get())))))).getOrDefault(false)) {
				change = NotionalAmountAfter(transactionReportInstruction, defaultValue).get();
			} else {
				change = abs.evaluate(MapperMaths.<BigDecimal, BigDecimal, BigDecimal>subtract(NotionalAmountAfter(transactionReportInstruction, defaultValue), NotionalAmountBefore(transactionReportInstruction, defaultValue)).get());
			}
			
			return change;
		}
		
		@Override
		protected MapperS<? extends BusinessEvent> businessEvent(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue) {
			return MapperS.of(transactionReportInstruction).<WorkflowStep>map("getOriginatingWorkflowStep", _transactionReportInstruction -> _transactionReportInstruction.getOriginatingWorkflowStep()).<BusinessEvent>map("getBusinessEvent", workflowStep -> workflowStep.getBusinessEvent());
		}
		
		@Override
		protected MapperS<? extends PayoutLegWithAuxiliary> payoutBefore(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue) {
			final MapperS<TradableProduct> thenArg = MapperS.of(beforeTradeForEvent.evaluate(transactionReportInstruction)).<TradableProduct>map("getTradableProduct", trade -> trade.getTradableProduct());
			return thenArg
				.mapSingleToItem(item -> MapperS.of(payoutLeg1.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get(), item.<TradeLot>mapC("getTradeLot", tradableProduct -> tradableProduct.getTradeLot()).get(), MapperS.of(transactionReportInstruction).<ReportableInformation>map("getReportableInformation", _transactionReportInstruction -> _transactionReportInstruction.getReportableInformation()).get())));
		}
		
		@Override
		protected MapperS<BigDecimal> NotionalAmountBefore(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue) {
			return MapperS.of(notionalAmountFormat.evaluate(payoutBefore(transactionReportInstruction, defaultValue).get(), defaultValue));
		}
		
		@Override
		protected MapperS<BigDecimal> NotionalAmountAfter(TransactionReportInstruction transactionReportInstruction, BigDecimal defaultValue) {
			return MapperS.of(notionalAmountLeg1.evaluate(transactionReportInstruction, defaultValue));
		}
	}
}
